"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _symbol = require("babel-runtime/core-js/symbol");

var _symbol2 = _interopRequireDefault(_symbol);

var _graphql = require("graphql");

var _queryFromResolveData = require("../queryFromResolveData");

var _queryFromResolveData2 = _interopRequireDefault(_queryFromResolveData);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nullableIf = function nullableIf(condition, Type) {
  return condition ? Type : new _graphql.GraphQLNonNull(Type);
};

exports.default = function PgColumnsPlugin(builder, _ref) {
  var inflection = _ref.pgInflection;

  builder.hook("GraphQLObjectType:fields", function (fields, _ref2, _ref3) {
    var extend = _ref2.extend,
        gqlTypeByTypeId = _ref2.pgGqlTypeByTypeId,
        introspectionResultsByKind = _ref2.pgIntrospectionResultsByKind,
        sql = _ref2.pgSql,
        pg2gql = _ref2.pg2gql,
        getAliasFromResolveInfo = _ref2.getAliasFromResolveInfo,
        pgTweakFragmentForType = _ref2.pgTweakFragmentForType;
    var _ref3$scope = _ref3.scope,
        isPgRowType = _ref3$scope.isPgRowType,
        isPgCompoundType = _ref3$scope.isPgCompoundType,
        table = _ref3$scope.pgIntrospection,
        fieldWithHooks = _ref3.fieldWithHooks;

    if (!(isPgRowType || isPgCompoundType) || !table || table.kind !== "class") {
      return fields;
    }
    return extend(fields, introspectionResultsByKind.attribute.filter(function (attr) {
      return attr.classId === table.id;
    }).reduce(function (memo, attr) {
      /*
      attr =
        { kind: 'attribute',
          classId: '6546809',
          num: 21,
          name: 'upstreamName',
          description: null,
          typeId: '6484393',
          isNotNull: false,
          hasDefault: false }
      */
      var fieldName = inflection.column(attr.name, table.name, table.namespaceName);
      memo[fieldName] = fieldWithHooks(fieldName, function (_ref4) {
        var getDataFromParsedResolveInfoFragment = _ref4.getDataFromParsedResolveInfoFragment,
            addDataGenerator = _ref4.addDataGenerator;

        var ReturnType = gqlTypeByTypeId[attr.typeId] || _graphql.GraphQLString;
        addDataGenerator(function (parsedResolveInfoFragment) {
          var alias = parsedResolveInfoFragment.alias;

          if (attr.type.type === "c") {
            return {
              pgQuery: function pgQuery(queryBuilder) {
                // json_build_object
                /*
                queryBuilder.select(
                  sql.identifier(queryBuilder.getTableAlias(), attr.name),
                  alias
                );
                */
                var resolveData = getDataFromParsedResolveInfoFragment(parsedResolveInfoFragment, ReturnType);
                var jsonBuildObject = (0, _queryFromResolveData2.default)(sql.identifier((0, _symbol2.default)()), // Ignore!
                sql.fragment`(${queryBuilder.getTableAlias()}.${sql.identifier(attr.name)})`, // The brackets are necessary to stop the parser getting confused, ref: https://www.postgresql.org/docs/9.6/static/rowtypes.html#ROWTYPES-ACCESSING
                resolveData, { onlyJsonField: true });
                queryBuilder.select(jsonBuildObject, alias);
              }
            };
          } else {
            return {
              pgQuery: function pgQuery(queryBuilder) {
                queryBuilder.select(pgTweakFragmentForType(sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(attr.name)}`, attr.type), alias);
              }
            };
          }
        });
        return {
          description: attr.description,
          type: nullableIf(!attr.isNotNull, ReturnType),
          resolve: function resolve(data, _args, _context, resolveInfo) {
            var alias = getAliasFromResolveInfo(resolveInfo);
            return pg2gql(data[alias], attr.type);
          }
        };
      });
      return memo;
    }, {}));
  });
  builder.hook("GraphQLInputObjectType:fields", function (fields, _ref5, _ref6) {
    var extend = _ref5.extend,
        gqlInputTypeByTypeId = _ref5.pgGqlInputTypeByTypeId,
        introspectionResultsByKind = _ref5.pgIntrospectionResultsByKind;
    var _ref6$scope = _ref6.scope,
        isPgRowType = _ref6$scope.isPgRowType,
        isPgCompoundType = _ref6$scope.isPgCompoundType,
        isPgPatch = _ref6$scope.isPgPatch,
        table = _ref6$scope.pgIntrospection,
        pgAddSubfield = _ref6$scope.pgAddSubfield;

    if (!(isPgRowType || isPgCompoundType) || !table || table.kind !== "class") {
      return fields;
    }
    return extend(fields, introspectionResultsByKind.attribute.filter(function (attr) {
      return attr.classId === table.id;
    }).reduce(function (memo, attr) {
      var fieldName = inflection.column(attr.name, table.name, table.namespaceName);
      memo[fieldName] = pgAddSubfield(fieldName, attr.name, attr.type, {
        description: attr.description,
        type: nullableIf(isPgPatch || !attr.isNotNull || attr.hasDefault, gqlInputTypeByTypeId[attr.typeId] || _graphql.GraphQLString)
      });
      return memo;
    }, {}));
  });
};
//# sourceMappingURL=PgColumnsPlugin.js.map